--[[
	utility.lua
		Utility functions for Bongos
--]]

--[[ Somewhat Higher Order Functions ]]--

--Iterates through all BongosBars, and performs the action specified
function Bongos_ForEachBar(action, arg1)
	local children = { BongosParent:GetChildren() };
	local i;
	
	for i in children do
		action(children[i], arg1);
	end
end

--takes a Bongos BarID, and performs the specified action on that bar
function Bongos_ForBarID(barID, action, arg1)
	if(not barID) then
		return;
	end
	
	if(barID == "all") then
		Bongos_ForEachBar(action, arg1);
	else
		local bar = BBar_IDToBar(barID);
		if(bar) then
			action(bar, arg1);
		end
	end
end

--[[ Other - Just useful stuff]]--

function Bongos_IsQuickMoveKeyDown()
	if( Bongos.quickMove) then
		if(Bongos.quickMove == 1 and IsShiftKeyDown() ) then
			return true;
		end
		if(Bongos.quickMove == 2 and IsControlKeyDown() ) then
			return true;
		end
		return (Bongos.quickMove == 3 and IsAltKeyDown() );
	end
	return false;
end

function Bongos_IsSelfCastHotkeyDown()
	if( Bongos.altCast) then
		if(Bongos.altCast == 1 and IsAltKeyDown() ) then
			return true;
		end
		if(Bongos.altCast == 2 and IsControlKeyDown() ) then
			return true;
		end
		return (Bongos.altCast == 3 and IsShiftKeyDown() );
	end
	return false;
end

--Update  Button's hotkey
function Bongos_UpdateHotkey(button, actionButtonType)
	if ( not actionButtonType ) then
		actionButtonType = "ACTIONBUTTON";
	end
	
	local keyBindingName = actionButtonType .. button:GetID();
	local key = string.upper(GetBindingText(GetBindingKey( keyBindingName ), "KEY_"));
	
	--hotkeys are shortened to one letter for Alt, Ctrl, Shift, and Num Pad
	if(key) then
		key = string.gsub (key, " ", "");
		key = string.gsub (key, "ALT%-", "A");
		key = string.gsub (key, "CTRL%-", "C");
		key = string.gsub (key, "SHIFT%-", "S");
		key = string.gsub (key, "NUMPAD", "N");
		key = string.gsub (key, "BACKSPACE", "BSpc");
		
		--extra hotkeys
		key = string.gsub (key, "HOME", "Hm");
		key = string.gsub (key, "END", "End");
		key = string.gsub (key, "INSERT", "Ins");
		key = string.gsub (key, "DELETE", "Del");
		key = string.gsub (key, "MIDDLEMOUSE", "M3");
		key = string.gsub (key, "MOUSEBUTTON4", "M4");
		key = string.gsub (key, "MOUSEBUTTON5", "M5");
		key = string.gsub (key, "MOUSEWHEELDOWN", "MwDn");
		key = string.gsub (key, "MOUSEWHEELUP", "MwUp");
		key = string.gsub (key, "PAGEDOWN", "PgDn");
		key = string.gsub (key, "PAGEUP", "PgUp"); 
	end
	
	 getglobal(button:GetName().."HotKey"):SetText( key );
end

--Print a chat message
function BMsg(msg)
	DEFAULT_CHAT_FRAME:AddMessage( (msg or "null"), 0,1,0.4);
end

function Bongos_AnchorTooltip(frame)
	GameTooltip:ClearAllPoints();
	
	if( frame:GetLeft() < ( UIParent:GetRight() / 2) ) then
		if(frame:GetTop() < (UIParent:GetTop() / 2) ) then
			GameTooltip:SetPoint("BOTTOMLEFT", frame, "TOPLEFT");
		else
			GameTooltip:SetPoint("TOPLEFT", frame, "BOTTOMRIGHT");
		end
	else
		if(frame:GetTop() < (UIParent:GetTop() / 2) ) then
			GameTooltip:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT");
		else
			GameTooltip:SetPoint("TOPRIGHT", frame, "BOTTOMLEFT");
		end
	end
end