--[[
	BMenuBar
		Scripts used for the Bongos MenuBar

	Saved Variables:
		Bongos.menu = {
			<All variables from BBar>
			space
				The spacing between action buttons, in pixels.  A nil value means that the bar is using default spacing
			rows
				How many rows the bar is organized into.
		}
--]]

--constants
local DEFAULT_SPACING = 2;
local DEFAULT_ROWS = 1;

function BMenuBar_Create()
	local bar = BBar_Create("BMenuBar", "menu");
	BMenuBarDragButton:SetScript("OnClick", function()
		if(arg1 == "RightButton") then
			BMenuBar_ShowMenu();
		end
	end);
	
	CharacterMicroButton:SetParent(BMenuBar);
	SpellbookMicroButton:SetParent(BMenuBar);
	TalentMicroButton:SetParent(BMenuBar);
	QuestLogMicroButton:SetParent(BMenuBar);
	SocialsMicroButton:SetParent(BMenuBar);
	WorldMapMicroButton:SetParent(BMenuBar);
	MainMenuMicroButton:SetParent(BMenuBar);
	HelpMicroButton:SetParent(BMenuBar);
	
	local alpha = BMenuBar:GetAlpha();
	if(alpha ~= 1) then
		CharacterMicroButton:SetAlpha(alpha);
		SpellbookMicroButton:SetAlpha(alpha);
		TalentMicroButton:SetAlpha(alpha);
		QuestLogMicroButton:SetAlpha(alpha);
		SocialsMicroButton:SetAlpha(alpha);
		WorldMapMicroButton:SetAlpha(alpha);
		MainMenuMicroButton:SetAlpha(alpha);
		HelpMicroButton:SetAlpha(alpha);
	end
	
	CharacterMicroButton:ClearAllPoints();
	CharacterMicroButton:SetPoint("TOPLEFT", BMenuBar, "TOPLEFT", 1, 20);
	
	if( BMenuBar:IsShown() ) then
		BMenuBar:Hide();
		BMenuBar:Show();
	end
	
	if(not BMenuBar:IsUserPlaced() ) then
		BMenuBar:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -224, 0);
	end
	
	BMenuBar_Build(Bongos.menu.rows, Bongos.menu.space);
	
	return bar;
end


--[[ Loading Functions ]]--
function BMenuBar_Load()
	BMenuBar_Create();
	
	UpdateTalentButton = BUpdateTalentButton;
end

--[[ UI Functions ]]--

--Layout function
function BMenuBar_Build(rows, space)
	if(not space or space == DEFAULT_SPACING) then
		Bongos.menu.space = nil;
		space = DEFAULT_SPACING;
	else
		Bongos.menu.space = space;
	end
	
	if(not rows or rows == DEFAULT_ROWS) then
		Bongos.menu.rows = nil;
		rows = DEFAULT_ROWS;
	else
		Bongos.menu.rows = rows;
	end
	
	SpellbookMicroButton:ClearAllPoints();
	TalentMicroButton:ClearAllPoints();
	QuestLogMicroButton:ClearAllPoints();
	SocialsMicroButton:ClearAllPoints();
	WorldMapMicroButton:ClearAllPoints();
	MainMenuMicroButton:ClearAllPoints();
	HelpMicroButton:ClearAllPoints();
	
	local actSpace = space;
	if( rows ~= DEFAULT_ROWS ) then
		--vertical layout
		space = space - 24; --apparently the anchors are weird on the micro buttons, and need to be adjusted
		
		SpellbookMicroButton:SetPoint("TOP", CharacterMicroButton, "BOTTOM", 0, -space);
		TalentMicroButton:SetPoint("TOP", SpellbookMicroButton, "BOTTOM", 0, -space);
		QuestLogMicroButton:SetPoint("TOP", TalentMicroButton, "BOTTOM", 0, -space);
		SocialsMicroButton:SetPoint("TOP", QuestLogMicroButton, "BOTTOM", 0, -space);
		WorldMapMicroButton:SetPoint("TOP", SocialsMicroButton, "BOTTOM", 0, -space);
		MainMenuMicroButton:SetPoint("TOP", WorldMapMicroButton, "BOTTOM", 0, -space);
		HelpMicroButton:SetPoint("TOP", MainMenuMicroButton, "BOTTOM", 0, -space);
		
		BMenuBar:SetHeight(13 + (33 + actSpace) * 8 - actSpace);
		BMenuBar:SetWidth(32);
	else
		space = space - 4; --there's also excess space added on the horizontal
		--horizontal layout
		SpellbookMicroButton:SetPoint("LEFT", CharacterMicroButton, "RIGHT", space, 0);
		TalentMicroButton:SetPoint("LEFT", SpellbookMicroButton, "RIGHT", space, 0);
		QuestLogMicroButton:SetPoint("LEFT", TalentMicroButton, "RIGHT", space, 0);
		SocialsMicroButton:SetPoint("LEFT", QuestLogMicroButton, "RIGHT", space, 0);
		WorldMapMicroButton:SetPoint("LEFT", SocialsMicroButton, "RIGHT", space, 0);
		MainMenuMicroButton:SetPoint("LEFT", WorldMapMicroButton, "RIGHT", space, 0);
		HelpMicroButton:SetPoint("LEFT", MainMenuMicroButton, "RIGHT", space, 0);
		
		BMenuBar:SetHeight(39);
		BMenuBar:SetWidth(14 + (24 + actSpace) * 8 - actSpace);
	end
end

--[[ Blizzard Function Replacements ]]--

--Prevents the talent button from always showing up even when the bar is hidden
function BUpdateTalentButton()
	if ( UnitLevel("player") < 10 ) then
		TalentMicroButton:Hide();
	else
		if(Bongos.menu and Bongos.menu.vis) then
			TalentMicroButton:Show();
		end
	end
end

--[[ Rightclick Menu Functions ]]--
local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenuHide");
	menu:SetText("Menu Bar");
	menu:SetWidth(220);
	menu:SetHeight(220);

	local verticalButton = CreateFrame("CheckButton", name .. "Vertical", menu, "BongosCheckButtonTemplate");
	verticalButton:SetScript("OnClick", function()
		if( this:GetChecked() ) then
			BMenuBar_Build(5, Bongos.menu.space );
		else
			BMenuBar_Build(nil, Bongos.menu.space );
		end
	end);
	verticalButton:SetPoint("TOP", name .. "Hide", "BOTTOM", 0, 4);
	verticalButton:SetText("Vertical");
	
	local spacingSlider = CreateFrame("Slider", name .. "Spacing", menu, "BongosSpaceSlider");
	spacingSlider:SetPoint("TOPLEFT", name .. "Vertical", "BOTTOMLEFT", 2, -10);
	spacingSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			BMenuBar_Build(Bongos.menu.rows, this:GetValue() );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	
	local scaleSlider = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider");
	scaleSlider:SetPoint("TOP", spacingSlider, "BOTTOM", 0, -24);
	
	local opacitySlider = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider");
	opacitySlider:SetPoint("TOP", scaleSlider, "BOTTOM", 0, -24);
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BMenuBar_ShowMenu()
	if(not BongosMenuBarMenu) then
		CreateConfigMenu("BongosMenuBarMenu");
	end
	
	BongosMenuBarMenu.onShow = 1;
	
	BongosMenuBarMenu.frame = this:GetParent();
	
	BongosMenuBarMenuHide:SetChecked(not Bongos.menu.vis);
	BongosMenuBarMenuVertical:SetChecked(Bongos.menu.rows);
	
	if(Bongos.menu.space) then
		BongosMenuBarMenuSpacing:SetValue(Bongos.menu.space);
	else
		BongosMenuBarMenuSpacing:SetValue(DEFAULT_SPACING);
	end
	
	BongosMenuBarMenuScale:SetValue( BMenuBar:GetScale() * 100 );
	BongosMenuBarMenuOpacity:SetValue( BMenuBar:GetAlpha() * 100 );
	
	BMenu_ShowMenuForBar(BongosMenuBarMenu, BMenuBar);
	BongosMenuBarMenu.onShow = nil;
end